import datetime as dt
import re

from bs4 import BeautifulSoup
import yt_dlp


# embedding yt-dlp instead of CLI let's us hold on to the output filepaths
# probably way more trouble than it's worth and should just use yt-dlp CLI
#video_url = 'https://www.youtube.com/watch?v=WJaxFbdjm8c'
#!yt-dlp --write-auto-subs {video_url}


# I still have mixed feelings about using this helper class vs. just using the yt-dlp cli
class YoutubeHelper:
    def __init__(
        self, 
        video_url,
        ydl_opts = {
            #'outtmpl':{'default':"ytdlp_content.%(ext)s"},
            'writeautomaticsub':True,
            'subtitlesformat':'srv2/vtt'
            },
    ):
        self.url = video_url
        #with yt_dlp.YoutubeDL(ydl_opts) as ydl:
        self.ydl = yt_dlp.YoutubeDL(ydl_opts)
        self.info = self.ydl.extract_info(video_url, download=True)
        #self.subs = self.get_subtitles()
        #self.audio = self.get_audio()

    def get_subtitles(
        self,
        lang='en',
        fmt='vtt', # 'vtt','ttml','srv3','srv2','srv1','json3'
    ):
        cc_targets = self.info['automatic_captions'][lang]
        item = [item for item in cc_targets if item['ext']==fmt]
        try:
            assert len(item) > 0
        except AssertionError:
            print(
                f"Captions for language [{lang}] and format [{fmt}] not available. "
                "Please try a different language or subtitle format"
            )
            return
        return item[0]

def parse_timestamp(ts):
    t = dt.datetime.strptime(ts,"%H:%M:%S.%f")
    return dt.timedelta(
        hours=t.hour,
        minutes=t.minute,
        seconds=t.second,
        microseconds=t.microsecond,
        )

def vtt_to_token_timestamps(captions):

    all_word_starts_raw = []
    chunk_starts = {}
    for cap in captions:
      chunks = cap.text.split('\n')
      chunks_raw = cap.raw_text.split('\n')
      for c, cr in zip(chunks, chunks_raw):
          if not c.strip():
              continue
          if '<c>' in cr:
              cr=f"<{cap.start}>{cr}"
              all_word_starts_raw.append(cr)
              chunk_starts[c] = cap.start

    # write once, read never.
    pat = re.compile("<?(([0-9:.]{12})>(<c>){0,1}(.+?))<")

    token_start_times = []
    for line in all_word_starts_raw:
      starts_ = [
          {'ts':hit[1], 
           'tok':hit[3].strip(),
           'td':parse_timestamp(hit[1])
           } 
           for hit in re.findall(pat, line)]
      token_start_times.extend(starts_)

    return token_start_times

def srv2_to_token_timestamps(srv2_xml):
    srv2_soup = BeautifulSoup(srv2_xml, 'xml')
    return [
        {
         'ts':e['t'], 
         'tok':e.text,
         #'td':dt.timedelta(microseconds=int(e['t']))
         'td':dt.timedelta(milliseconds=int(e['t']))
         } 
        for e in srv2_soup.find_all('text')
        if e.text.strip() 
    ]


from pathlib import Path
import random
import string
import subprocess
import textwrap

import pandas as pd
from PIL import Image, ImageDraw, ImageFont

def gpu_info():
    outv = subprocess.run([
        'nvidia-smi',
            # these lines concatenate into a single query string
            '--query-gpu='
            'timestamp,'
            'name,'
            'utilization.gpu,'
            'utilization.memory,'
            'memory.used,'
            'memory.free,'
            ,
         '--format=csv'
        ],
        stdout=subprocess.PIPE).stdout.decode('utf-8')

    header, rec = outv.split('\n')[:-1]
    return pd.DataFrame({' '.join(k.strip().split('.')).capitalize():v for k,v in zip(header.split(','), rec.split(','))}, index=[0]).T


def remove_punctuation(s):
    # https://stackoverflow.com/a/266162/819544
    return s.translate(str.maketrans('', '', string.punctuation))



def add_caption2image(
      image, 
      caption, 
      text_font='LiberationSans-Regular.ttf', 
      font_size=20,
      fill_color=(255, 255, 255),
      stroke_color=(0, 0, 0), #stroke_fill
      stroke_width=2,
      align='center',
      ):
    # via https://stackoverflow.com/a/59104505/819544
    wrapper = textwrap.TextWrapper(width=50) 
    word_list = wrapper.wrap(text=caption) 
    caption_new = ''
    for ii in word_list[:-1]:
        caption_new = caption_new + ii + '\n'
    caption_new += word_list[-1]

    draw = ImageDraw.Draw(image)

    # Download the Font and Replace the font with the font file. 
    font = ImageFont.truetype(text_font, size=font_size)
    w,h = draw.textsize(caption_new, font=font, stroke_width=stroke_width)
    W,H = image.size
    x,y = 0.5*(W-w),0.90*H-h
    draw.text(
        (x,y), 
        caption_new,
        font=font,
        fill=fill_color, 
        stroke_fill=stroke_color,
        stroke_width=stroke_width,
        align=align,
    )

    return image


def rand_str(n_char=5):
    return ''.join(random.choice(string.ascii_lowercase) for i in range(n_char))


def save_frame(
    img: Image,
    idx:int=0,
    root_path=Path('./frames'),
    name=None,
):
    root_path.mkdir(parents=True, exist_ok=True)
    if name is None:
        name = rand_str()
    outpath = root_path / f"{idx}-{name}.png"
    img.save(outpath)
    return str(outpath)